
#ifndef MODULE_durabilityModule2_HEADER
#define MODULE_durabilityModule2_HEADER

#include "c_base.h"
#include "c_misc.h"
#include "c_sync.h"
#include "c_collection.h"
#include "c_field.h"
#include "kernelModule.h"

typedef struct c_time d_timestamp;

typedef struct c_time d_quality;

typedef c_string d_name;

typedef c_string d_partition;

typedef c_string d_topic;

struct d_networkAddress_s {
    c_ulong systemId;
    c_ulong localId;
    c_ulong lifecycleId;
};

typedef enum d_serviceState {
    D_STATE_INIT,
    D_STATE_DISCOVER_FELLOWS_GROUPS,
    D_STATE_DISCOVER_LOCAL_GROUPS,
    D_STATE_DISCOVER_PERSISTENT_SOURCE,
    D_STATE_INJECT_PERSISTENT,
    D_STATE_FETCH_INITIAL,
    D_STATE_COMPLETE,
    D_STATE_FETCH,
    D_STATE_ALIGN,
    D_STATE_FETCH_ALIGN,
    D_STATE_TERMINATING,
    D_STATE_TERMINATED
} d_serviceState;

typedef enum d_completeness {
    D_GROUP_KNOWLEDGE_UNDEFINED,
    D_GROUP_UNKNOWN,
    D_GROUP_INCOMPLETE,
    D_GROUP_COMPLETE
} d_completeness;

typedef enum d_durabilityKind {
    D_DURABILITY_VOLATILE,
    D_DURABILITY_TRANSIENT_LOCAL,
    D_DURABILITY_TRANSIENT,
    D_DURABILITY_PERSISTENT,
    D_DURABILITY_ALL
} d_durabilityKind;

typedef enum d_alignmentKind {
    D_ALIGNEE_INITIAL,
    D_ALIGNEE_LAZY,
    D_ALIGNEE_ON_REQUEST
} d_alignmentKind;

typedef enum d_mergePolicy {
    D_MERGE_IGNORE,
    D_MERGE_MERGE,
    D_MERGE_DELETE,
    D_MERGE_REPLACE
} d_mergePolicy;

struct d_mergeState_s {
    d_name role;
    c_ulong value;
};

struct d_message_s {
    struct d_networkAddress_s addressee;
    struct d_networkAddress_s senderAddress;
    enum d_serviceState senderState;
    d_timestamp productionTimestamp;
    c_ulong sequenceNumber;
};

struct d_groupsRequest_s {
    struct d_message_s parentMsg;
    d_partition partition;
    d_topic topic;
};

struct d_statusRequest_s {
    struct d_message_s parentMsg;
};

struct d_status_s {
    struct d_message_s parentMsg;
};

struct d_nameSpacesRequest_s {
    struct d_message_s parentMsg;
};

struct d_nameSpaces_s {
    struct d_message_s parentMsg;
    c_string name;
    c_string partitions;
    enum d_durabilityKind durabilityKind;
    c_bool aligner;
    enum d_alignmentKind alignmentKind;
    d_quality initialQuality;
    c_ulong total;
    struct d_networkAddress_s master;
    c_bool isComplete;
    c_bool masterConfirmed;
    struct d_mergeState_s state;
    c_sequence mergedStates /*C_SEQUENCE<durabilityModule2::d_mergeState_s>*/;
    c_ulong mergedStatesCount;
};

struct d_sampleRequest_s {
    struct d_message_s parentMsg;
    d_partition partition;
    d_topic topic;
    enum d_durabilityKind durabilityKind;
    d_timestamp requestTime;
    c_bool withTimeRange;
    d_timestamp beginTime;
    d_timestamp endTime;
    struct d_networkAddress_s source;
    c_string filter;
    c_sequence filterParams /*C_SEQUENCE<c_string>*/;
    c_ulong filterParamsCount;
    c_long maxSamples;
    c_long maxInstances;
    c_long maxSamplesPerInstance;
};

struct d_newGroup_s {
    struct d_message_s parentMsg;
    d_partition partition;
    d_topic topic;
    enum d_completeness completeness;
    enum d_durabilityKind durabilityKind;
    d_quality quality;
    c_ulong alignerCount;
};

struct d_sampleChainBead {
    c_sequence value /*C_SEQUENCE<c_octet>*/;
    c_long size;
};

struct d_sampleChainTrailer {
    c_ulong nrSamples;
    enum d_completeness completeness;
};

typedef enum content {
    BEAD,
    LINK
} content;

struct messageContent {
    enum content _d;
    union {
        struct d_sampleChainBead bead;
        struct d_sampleChainTrailer link;
    } _u;
};

struct d_sampleChain_s {
    struct d_message_s parentMsg;
    c_string partition;
    c_string topic;
    enum d_durabilityKind durabilityKind;
    c_sequence addressees /*C_SEQUENCE<durabilityModule2::d_networkAddress_s>*/;
    c_ulong addresseesCount;
    struct d_networkAddress_s source;
    struct messageContent msgBody;
};

struct d_deleteData_s {
    struct d_message_s parentMsg;
    d_timestamp actionTime;
    c_string partitionExpr;
    c_string topicExpr;
};

C_CLASS(d_sample);

C_STRUCT(d_sample) {
    c_voidp instance;
    c_voidp newer;
    d_sample older;
};

C_CLASS(d_sampleTemplate);

C_STRUCT(d_sampleTemplate) {
    C_EXTENDS(d_sample);
    v_message message;
};

C_CLASS(d_instance);

C_STRUCT(d_instance) {
    c_long count;
    c_long messageCount;
    c_voidp oldest;
    c_ulong state;
};

C_CLASS(d_instanceTemplate);

C_STRUCT(d_instanceTemplate) {
    C_EXTENDS(d_instance);
    d_sampleTemplate newest;
};

C_CLASS(d_topicInfo);

C_STRUCT(d_topicInfo) {
    c_type dataType;
    c_string instanceKeyExpr;
    c_type instanceType;
    c_string keyExpr;
    c_type keyType;
    c_type messageType;
    c_string name;
    v_topicQos qos;
    c_type sampleType;
    c_string typeName;
};

C_CLASS(d_groupInfo);

C_STRUCT(d_groupInfo) {
    enum d_completeness completeness;
    c_voidp kernel;
    d_quality quality;
    c_set instances;
    d_partition partition;
    d_topicInfo topic;
};

C_CLASS(d_nameSpaceInfo);

C_STRUCT(d_nameSpaceInfo) {
    c_bool complete;
    d_name name;
};

C_CLASS(d_storeMMFKernel);

C_STRUCT(d_storeMMFKernel) {
    c_set backup;
    c_set groups;
    c_set nameSpaces;
    c_set topics;
};

c_bool
loaddurabilityModule2(
    c_base base);


#endif /* MODULE_durabilityModule2_HEADER */

