#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e6|x1> <debug|prof> [tar]"
}


# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	ddsextraqual="release"
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
	ddsextraqual="dev"
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	ddsextraqual="release"
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}


get_platform_name()
{
    if [[ "${basequal}" == *"x"* ]]
    then 
      OS1="Linuxppc"
      ddsflvr="PPC.linux2.6"
#      if [[ "${basequal}" != *"ppc"* ]]
#      then
#	basequal="${basequal}:ppc32"
#	set_qualifiers
#      fi
    else
      OS1=`uname`
      ddsflvr=`uname -p`.`lowercase \`uname\` ``uname -r |cut -f1,2 -d"."`
    fi
    
    flvr=`ups flavor`
    
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    elif [ ${OS1} = "Linuxppc" ]
    then
	flvr="Linuxppc"
    fi
    
    if [ ${OS1} = "Linuxppc" ]
    then
      OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osname ${OS1}`
      plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat ${OS1}`
    else
      OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
      plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`
    fi
    
    
    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi
    
    build_flavor=`echo ${OS}-${qualdir}|sed -e 's/-/./g'`
    
    pkgdir=${product_dir}/${package}/${pkgver}/${build_flavor}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${build_flavor}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}


lowercase(){
echo "$1" | sed "y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/"
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=opensplicedds
origpkgver=v5_3_2
pkgver=${origpkgver}
ssibuildshims_version=v0_08_01
cmake_version=v3_0_1
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=opensplicedds-${pkgdotver}.tgz

get_this_dir

get_ssibuildshims

set_qualifiers
 
get_platform_name

make_ups_product_tarbz2(){
   echo ""
   echo "OK - building ${OS} tarball for ${package} ${pkgver} ${qualdir}"
   set -x
    
   saved_dir=`pwd`
   cd ${product_dir}
    
   tar cjf ${package}-${pkgdotver}-${build_flavor}.tar.bz2 \
	${package}/${pkgver}.version/${flvr}*${extraqual}* \
        ${package}/${pkgver}/${build_flavor} \
        ${package}/${pkgver}/ups 

   cd ${saved_dir}
}

#if false; then #skip code

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_ups_product_tarbz2
   exit 0
fi
  

echo "building ${package} for ${OS}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) extra_command="-DCMAKE_BUILD_TYPE=Debug";;
opt)   extra_command="-DCMAKE_BUILD_TYPE=Release";;
prof)   extra_command="-DCMAKE_BUILD_TYPE=Release";;
* )
   echo "ERROR: please specify debug, opt, or prof"
   usage
   exit 1
;;
esac

fakedb=${product_dir}/${package}/${pkgver}/fakedb

rm -rf ${sourcedir} ${fakedb} ${pkgdir} ${blddir}

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi


#find odlpp compiler 

if [[ "${basequal}" == *"x"* ]]
then 
  ups_flvr=`ups flavor`
  setup -B ${package} ${pkgver} -q  `ups list -aK+ ${package} ${pkgver} -f ${ups_flvr} |grep prof |cut -d' ' -f4` || die "ERROR: setup -B ${package} ${pkgver} failed"  
  export OPENSPLICEDDS_X86INST_DIR=${OPENSPLICEDDS_BASE}
  unsetup ${SETUP_OPENSPLICEDDS}
  setup -B nova_crosscompiler  v4_5_0  -q ppc603 -f Linux64bit+2.6-2.5
fi

# declare now so we can setup
# fake ups declare


${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}  ${flvr}

setup -B ${package} ${pkgver} -q ${fullqual} -f ${flvr} -z ${fakedb}:${PRODUCTS} || die "ERROR: fake setup failed"

# We need cmake
setup -B cmake ${cmake_version} || die "ERROR: setup -B cmake ${cmake_version} failed"

if [[ "${basequal}" == e[45] ]]
then
  cxxflg="-gdwarf-2 -Wno-deprecated-declarations -std=c++11"
  extra_command="${extra_command} -DOPENSPLICEDDS_BUILD_CXXSTD=ON"
elif [[ "${basequal}" == e6 ]]
then
  cxxflg="-gdwarf-2 -Wno-deprecated-declarations -std=c++1y"
  extra_command="${extra_command} -DOPENSPLICEDDS_BUILD_CXXSTD=ON"
elif [[ "${basequal}" == x1 ]]
then
  cxxflg=" "
  extra_command="${extra_command} -DOPENSPLICEDDS_BUILD_CXXSTD=ON"
else
  die "Qualifier $basequal not recognized."
fi

export CETCXXFLG=${cxxflg}

set -x
cd ${sourcedir}


tar xf ${tardir}/${pkgtarfile}

cd ${sourcedir}/${pkgdotver}/opensplicedds

export OSPL_HOME=`pwd`

./configure  ${ddsflvr}-${ddsextraqual}
(( $? == 0 )) || die "ERROR: configure failed."

if [ -n ${OPENSPLICEDDS_X86INST_DIR} ]
then
  echo "export PATH=${OPENSPLICEDDS_X86INST_DIR}/bin:\$PATH" >> ${sourcedir}/${pkgdotver}/opensplicedds/envs-${ddsflvr}-${ddsextraqual}.sh
  echo "export LD_LIBRARY_PATH=${OPENSPLICEDDS_X86INST_DIR}/lib:\$LD_LIBRARY_PATH" >> ${sourcedir}/${pkgdotver}/opensplicedds/envs-${ddsflvr}-${ddsextraqual}.sh  
fi


SAVED_PATH=${PATH}

source  ${sourcedir}/${pkgdotver}/opensplicedds/envs-${ddsflvr}-${ddsextraqual}.sh
 

make || die "ERROR: make failed."

make install || die "ERROR: make install failed."

export PATH=${SAVED_PATH}


mkdir -p ${pkgdir}

cp -ra  $(dirname  `find ${sourcedir}/${pkgdotver}/opensplicedds/install/HDE/ -name release.com`)/* ${pkgdir}
cp -ra 	${sourcedir}/${pkgdotver}/opensplicedds/exec/${ddsflvr}-${ddsextraqual}/odlpp ${pkgdir}/bin

set +x

#fi #if false; then #skip code


if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
if [[ "${basequal}" == *"x"* ]]
then 
  pq=+`echo ${fullqual} | sed -e 's/:/:+/g'`
  ups declare ${package} ${pkgver} -r ${package}/${pkgver} -f ${flvr} -m opensplicedds.table -q ${pq} -z ${product_dir}
else
  ${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} ${flvr}
fi  


# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
pq=+`echo ${fullqual} | sed -e 's/:/:+/g'`
setup ${package} ${pkgver} -f ${flvr} -q ${pq} -z ${product_dir}:${PRODUCTS}
echo setup ${package} ${pkgver} -f ${flvr} -q ${pq} -z ${product_dir}:${PRODUCTS}
echo "opensplicedds is installed at ${OPENSPLICEDDS_BASE_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_ups_product_tarbz2
   exit 0
fi


exit 0
